% % =======================================================================
% % This script estimates a partially identified IV model using the
% % "semi-analytic" approach in Chan and Tobias (2015).
% %
% % output: moments is a 12x2 matrix that contains the post. mean and 
% %         std. dev. of all the partially identified parameters, namely
% %         coefficients associated with the intercept, MomBMI, DadBMI,
% %         FamilyIncome, HighSchool, ALevel, Degree, Union, Married, 
% %         BMI, the error correlation (rhouv) and the error variance in
% %         the outcome equation (sigu2)
% %
% % See:
% % Chan, J.C.C. and Tobias, J.L. (2015). "Priors and Posterior
% % Computation in Linear Endogenous Variables Models with Imperfect
% % Instruments," Journal of Applied Econometrics, 30(4), 650-674
% %
% % =======================================================================

clear all; clc;
% priorflag
% 1: normal for slope coefficients, inverse-Wishart for cov matrix
% 2: normal for slope coefficients with the restrictions
%  |alp_i|<gamub|beta_i|, inverse-Wishart for cov matrix 
priorflag = 1;
nloop = 51000;
burnin = 1000;

%% load data
load sim_data;
N = size(BMI,1);
Z = [ones(N,1) famincome highschool ALevel degree union ...
    married mombmi dadbmi]; % covariates in treatment eq
X = [ones(N,1) mombmi dadbmi jobtenure jobtenure.^2 experience ...
    experience.^2 famincome highschool ALevel degree union married ...
    momdegree mommanprof daddegree dadmanprof]; % covariates in outcome eq
na = size(X,2);
nb = size(Z,2);
n = na + nb + 1;
Ytilde = zeros(2*N,1);
Xtilde = zeros(2*N,n);
Xtilde(1:2:2*N,1:na) = X;
Xtilde(1:2:2*N,na+1) = BMI;
Xtilde(2:2:2*N,na+2:end) = Z;
Ytilde(1:2:2*N,1) = lnwage;
Ytilde(2:2:2*N,1) = BMI;
Xtilde = sparse(Xtilde);

%% prior
delta = .008;
gamub = -.025;
    % Vtheta(1,1) and Vtheta(na+2,na+2) are the variances of the intercepts
    % Vtheta(2,2) and Vtheta(3,3) are the variances of the IVs
    % invVtheta is the inverse of Vtheta
invVtheta = speye(n); invVtheta(1,1) = 1/5; invVtheta(na+2,na+2) = 1/5;
invVtheta(2,2) = 1/delta^2; invVtheta(3,3) = 1/delta^2;
theta0 = sparse(n,1);
nu0 = 4; S0 = (nu0-3)*[1 0; 0 1]; 
prior = struct('invVtheta',invVtheta, 'nu0',nu0,'gamub',gamub);

%% initialize for storage
newnu = N + nu0;
store_a = zeros(nloop-burnin,9); % [sig_v beta0 beta_1 beta_2 a_0 a_1 a_2 b c]
store_theta = zeros(nloop-burnin,n); 
store_Sig = zeros(nloop-burnin,3);
store_rho = zeros(nloop-burnin,1);

start_time = clock;
%% initialize the Markov chain
rho = -.2+.4*rand;
Sig = eye(2);
Sig(1) = .8*.3*rand;  Sig(4) = 20;
Sig(1,2) = rho*sqrt(Sig(1)*Sig(4));
Sig(2,1) = Sig(1,2);
invSig = Sig\speye(2); 

disp('Starting MCMC.... ');
disp(' ' );
for loop = 1:nloop    
    % sample theta
    sumtemp = Xtilde'*kron(speye(N),invSig);
    invDtheta = invVtheta + sumtemp*Xtilde;
    dtheta = invVtheta*theta0 + sumtemp*Ytilde;
    thetahat = invDtheta\dtheta;
    cholinvD = chol(invDtheta,'lower');
    theta = thetahat + cholinvD'\randn(n,1);
    if priorflag == 2
        while abs(theta(2)) > abs(gamub*theta(end-1)) || abs(theta(3)) > abs(gamub*theta(end)) 
            theta = thetahat + cholinvD'\randn(n,1);
        end  
    end
    
    % sample Sig
    Yerr = reshape(Ytilde - Xtilde*theta,2,N)';
    newS = S0 + Yerr'*Yerr;
    Sig = iwishrnd(newS, newnu);
    invSig = Sig\speye(2);
    
    if loop > burnin
        j=loop-burnin;
        store_theta(j,:) = theta';
        store_Sig(j,:) = [Sig(1) Sig(2) Sig(4)];
        store_rho(j,1) = Sig(2)/sqrt(Sig(1)*Sig(4));
        tempsig = Sig(2)/Sig(4);
        store_a(j,:) = [Sig(4) theta(na+2) theta(end-1) theta(end)...
            theta(1)-tempsig*theta(na+2) theta(2)-tempsig*theta(end-1)...
            theta(3)-tempsig*theta(end) theta(na+1)+tempsig Sig(1)-Sig(2)^2/Sig(4)];        
    end
        
    if ( mod( loop, 5000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end
end

moments = zeros(12,2); % store the post. mean and std. dev. of  
                       % theta([1 2 3 8 9 10 11 12 13 na+1]), rho and sigu2
compute_gam; 
compute_alp_IV; 
compute_alp;  
compute_rhouv;
compute_sigu2;

moments(2:3,1) = [Ealp1 Ealp2]';
moments([1 4:9],1) = store_Ealpi(:,1);
moments(10,1) = Egam;
moments(11:12,1) = [Erhouv Esigu2]';
moments(2:3,2) = [alp1std alp2std];
moments([1 4:9],2) = store_Ealpi(:,2);
moments(10,2) = gamstd;
moments(11:12,2) = [rhouvstd sigu2std]';


disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );
